/***********************************************************************
SubMenu - Subclass of RowColumn that only contains buttons and acts as a
sub-menu of another pop-up menu. This class replicates Menu's API but is
not related because it implies a different selection strategy in widget
managers.
Copyright (c) 2007-2008 Oliver Kreylos

This file is part of the GLMotif Widget Library (GLMotif).

The GLMotif Widget Library is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GLMotif Widget Library is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with the GLMotif Widget Library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
***********************************************************************/

#ifndef GLMOTIF_SUBMENU_INCLUDED
#define GLMOTIF_SUBMENU_INCLUDED

#include <Misc/CallbackData.h>
#include <Misc/CallbackList.h>
#include <GLMotif/Button.h>
#include <GLMotif/RowColumn.h>

namespace GLMotif {

class SubMenu:public RowColumn
	{
	/* Embedded classes: */
	public:
	class CallbackData:public Misc::CallbackData // Base class for callback data sent by submenu widgets
		{
		/* Elements: */
		public:
		SubMenu* menu; // Pointer to the submenu widget that caused the event
		
		/* Constructors and destructors: */
		CallbackData(SubMenu* sMenu)
			:menu(sMenu)
			{
			}
		};
	
	class EntrySelectCallbackData:public CallbackData // Class for callback data sent when a menu entry is selected
		{
		/* Elements: */
		public:
		Button* selectedButton; // Pointer to the button causing the event
		Button::SelectCallbackData* selectCallbackData; // Callback data generated by the button event
		
		/* Constructors and destructors: */
		EntrySelectCallbackData(SubMenu* sMenu,Button* sSelectedButton,Button::SelectCallbackData* sSelectCallbackData)
			:CallbackData(sMenu),selectedButton(sSelectedButton),selectCallbackData(sSelectCallbackData)
			{
			}
		};
	
	/* Elements: */
	protected:
	Misc::CallbackList entrySelectCallbacks; // List of callbacks to be called when a submenu entry is selected
	
	/* Protected methods: */
	static void childrenSelectCallbackWrapper(Misc::CallbackData* callbackData,void* userData); // Callback that is called when a child is selected
	
	/* Constructors and destructors: */
	public:
	SubMenu(const char* sName,Container* sParent,bool manageChild =true);
	
	/* Methods inherited from Container: */
	virtual void addChild(Widget* newChild);
	
	/* New methods: */
	void addEntry(const char* newEntryLabel); // Adds a new submenu entry button with the given label
	Misc::CallbackList& getEntrySelectCallbacks(void) // Returns list of entry select callbacks
		{
		return entrySelectCallbacks;
		}
	int getEntryIndex(const Button* entry) const; // Returns the index of the given submenu entry, or -1 if the button is not a child of the menu
	};

}

#endif
