/***********************************************************************
Doom3ModelNode - Class for nodes to render static models using Doom3's
lighting model.
Copyright (c) 2010-2021 Oliver Kreylos

This file is part of the Simple Scene Graph Renderer (SceneGraph).

The Simple Scene Graph Renderer is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The Simple Scene Graph Renderer is distributed in the hope that it will
be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with the Simple Scene Graph Renderer; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
***********************************************************************/

#ifndef SCENEGRAPH_DOOM3MODELNODE_INCLUDED
#define SCENEGRAPH_DOOM3MODELNODE_INCLUDED

#include <SceneGraph/FieldTypes.h>
#include <SceneGraph/GraphNode.h>
#include <SceneGraph/Doom3DataContextNode.h>

/* Forward declarations: */
namespace SceneGraph {
class Doom3Model;
}

namespace SceneGraph {

class Doom3ModelNode:public GraphNode
	{
	/* Embedded classes: */
	public:
	typedef SF<Doom3DataContextNodePointer> SFDoom3DataContextNode;
	
	/* Elements: */
	static const char* className; // The class's name
	
	/* Fields: */
	public:
	SFDoom3DataContextNode dataContext;
	SFString model;
	
	/* Derived state: */
	protected:
	Doom3Model* mesh; // Pointer to the triangle mesh object
	
	/* Constructors and destructors: */
	public:
	Doom3ModelNode(void);
	virtual ~Doom3ModelNode(void);
	
	/* Methods from class Node: */
	virtual const char* getClassName(void) const;
	virtual void parseField(const char* fieldName,VRMLFile& vrmlFile);
	virtual unsigned int update(void);
	
	/* Methods from class GraphNode: */
	virtual Box calcBoundingBox(void) const;
	virtual void glRenderAction(GLRenderState& renderState) const;
	};

}

#endif
